// File: FW_PillAlarm.ino
// 2015 jan@breem.nl
//
//  http://86.91.251.42/JanBeeProjects/TFT_Touch2p8inch/TFT-Touch8p2.htm 
//
// Target: Arduino Mega 2560 with 2.8" TFT Resistive TouchScreen
// 
// I need to take certain medicines at breakfast and dinner time, and I found myself 
// forgetting it to often. So I wrote this Pill Alarm application.
//
// Connect a self-oscillating beeper device between pin A8(+) and GND, next to A15 on the Mega.
// After the "Armed" time has passed a small beep sounds every minute and
// after the "Alarm" time the beeps sound every second.
// After Power On you must set the unit to the correct time-of-the-day with the + and - buttons.
//
// Suggestions for "ToDo":
// - add a DCF77 receiver and code so that the unit will find the correct time by itself.
// - add features to manually set the Armed and Alarm times and store them in EEPROM. (it's hardcoded now) 
//

#include <SPI.h>
#include <SeeedTouchScreen.h> 
#include "TFTv2.h"
#include "LibTFT2p8.h"
#include "controls.h"

enum t_State {stIdle, stArmed, stAlarm};
byte hour, minute, second, BeepTime;
bool TimeChanged;

t_State State;

//******************************************************************************************************
void ReDrawButtStop (void)
{
  switch (State)
  {
  case stIdle:  
    DrawButtStop (240, 60, GREEN, TouchStop, false);
    break;
  case stArmed:
    DrawButtStop (240, 60, CYAN, TouchStop, false);
    break;
  case stAlarm:
    DrawButtStop (240, 60, RED, TouchStop, false);
    break;
  }
}

//******************************************************************************************************
void DrawButtons (void)
{
  ClearScreen();
  DrawButtStop (240, 60, GREEN, TouchStop, true);
  DrawButtPlus (20, 100, GRAY1, TouchHourPlus, true);
  DrawButtMinus (20, 20, GRAY1, TouchHourMinus, true);
  DrawButtPlus (120, 100, GRAY1, TouchMinutePlus, true);
  DrawButtMinus (120, 20, GRAY1, TouchMinuteMinus, true);
}

//******************************************************************************************************
void TouchHourPlus (void)
{
  BeepTime = 10;  // ms
  if (hour < 23) hour++; else hour = 0;
  TimeChanged = true;
}

//******************************************************************************************************
void TouchHourMinus (void)
{
  BeepTime = 10;  // ms
  if (hour > 0) hour--; else hour = 23;
  TimeChanged = true;
}

//******************************************************************************************************
void TouchMinutePlus (void)
{
  BeepTime = 10;  // ms
  if (minute < 59) minute++; else minute = 0; 
  TimeChanged = true;
}

//******************************************************************************************************
void TouchMinuteMinus (void)
{
  BeepTime = 10;  // ms
  if (minute > 0) minute--; else minute = 59;
  TimeChanged = true;
}

//******************************************************************************************************
void TouchStop (void)
{
  BeepTime = 10;  // ms
  State = stIdle;
  ReDrawButtStop();
}

//************************************************************************************************
ISR(TIMER5_COMPA_vect) // Called at 1 kHz
{
  static int ms;
  if (BeepTime > 0) {PORTK |= 0x01; BeepTime--;} else PORTK &= ~0x01; // pin A8 on Arduino Mega 
  if (++ms >= 1000)
  {
    ms = 0; 
    if (++second >= 60)
    {
      second = 0;
      if (++minute >= 60)
      {
        minute = 0;
        if (++hour >= 24) hour = 0;
       }
      if (State == stArmed) BeepTime = 20; // each minute
    }
    TimeChanged = true;
    if (State == stAlarm) BeepTime = 10; // each second
  }
}

//******************************************************************************************************
void setup()
{
  tft_init_LibTFT2p8();
  
  // Configure 16 bit Timer 5 for 1 kHz interrupt frequency
  TCCR5A = 0x00;  // Waveform mode 4  no OCR outputs
  TCCR5B = 0x0A;  // Internal clock, prescaler 8 for 2MHz, WaveformMode 4
  OCR5A = 2000;   // Interrupt at 1 kHz
  TIMSK5 = 0x02;  // Interrupt on OCRA match 
  
  DrawButtons();  
}

//******************************************************************************************************
void loop()
{
  char str[10]; 
  int dummy = tft_DetectTouch(); // this calls the callback function
  if (TimeChanged) 
    {
      sprintf(str, "%02d:%02d:%02d", hour, minute, second);
      tft_DrawString (str, 20, 180, 4, YELLOW, BLACK);
      TimeChanged = false;
    } 
    
  // Here the Alarm Time settings are hard-coded. A future release will have GUI based settings  
  if (hour == 8 && minute == 0 && second == 0) {State = stArmed; ReDrawButtStop();}
  if (hour == 9 && minute == 0 && second == 0 && State == stArmed) {State = stAlarm; ReDrawButtStop();}
  if (hour == 17 && minute == 0 && second == 0) {State = stArmed; ReDrawButtStop();}
  if (hour == 19 && minute == 0 && second == 0 && State == stArmed) {State = stAlarm; ReDrawButtStop();}
   
  delay(200); // Avoid calling tft_DetectTouch to often, it may cause multiple events in a single touch
}

